 
/* 
 *      (c) Copyright 1991 Sun Microsystems, Inc. Sun design patents 
 *      pending in the U.S. and foreign countries. See LEGAL NOTICE 
 *      file for terms of the license.
 */

#include "../token/XtmtDefs.h"	/*
				 * This must be before Intrinsic.h so 
				 * function prototype names are redefined
				 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/Form.h>
#include <Xol/OpenLook.h>
#include <Xol/ControlAre.h>
#include <Xol/CheckBox.h>
#include <Xol/OblongButt.h>
#include <Xol/StaticText.h>
#include <Xol/Slider.h>
#include <Xol/Stub.h>
#include <Xol/Nonexclusi.h>
#include <thread.h>

#include "../token/XtToken.h"	/* 
				 * XtToken.h is only necessary because 
				 * OLIT entry points are not protected.
				 */



Widget	toplevel;
Widget	fill_item;
Widget	color_item;
Widget	linemessage, rectanglemessage, circlemessage;
Widget	canvas;

Display *dpy;
Window win;
unsigned long	fg, bg;

int sleep_time=5;
void change_sleep_time();

Widget create_slider();
void quit_callback();
void rectangle_thread();
void circle_thread();
void line_thread();
void random_coord();

thread_id_t line_t, rectangle_t, circle_t;


main(argc, argv)
	int             argc;
	char           *argv[];
{
	Widget          shell, form, controlarea;
	Widget          sleep_item;
	Widget          tmp;


	XtInitializeToken(); /* OLIT entry points are not protected */
	toplevel = OlInitialize(argv[0], "ex1", NULL, 0, &argc, argv);
	XtInitializeTokenDispatch(); /* OLIT entry points are not protected */

	form = XtCreateManagedWidget("form", formWidgetClass,
				     toplevel, NULL, 0);

	controlarea = XtCreateManagedWidget("controlarea", 
					    controlAreaWidgetClass,
					    form, NULL, 0);

	tmp = XtCreateManagedWidget("quitbutton", oblongButtonWidgetClass,
				    controlarea, NULL, 0);
	XtAddCallback(tmp, XtNselect, quit_callback, NULL);


	sleep_item = create_slider("sleepslider", controlarea, NULL, 0);

	XtAddCallback(XtNameToWidget(sleep_item, "sleepslider"),
		      XtNsliderMoved, change_sleep_time, NULL);

	fill_item = XtCreateManagedWidget("fill", checkBoxWidgetClass,
					  controlarea, NULL, 0);

	color_item = XtCreateManagedWidget("color", checkBoxWidgetClass,
					   controlarea, NULL, 0);

	tmp = XtCreateManagedWidget("messages", controlAreaWidgetClass,
				    form, NULL, 0);

	linemessage = XtCreateManagedWidget("lines", staticTextWidgetClass,
					    tmp, NULL, 0);

	rectanglemessage = XtCreateManagedWidget("rectangles", 
						 staticTextWidgetClass,
						 tmp, NULL, 0);

	circlemessage = XtCreateManagedWidget("circles", staticTextWidgetClass,
					      tmp, NULL, 0);

	canvas = XtCreateManagedWidget("canvas", stubWidgetClass,
				       form, NULL, 0);

	XtRealizeWidget(toplevel);

	dpy = XtDisplay(canvas);
	win = XtWindow(canvas);

	bg = BlackPixel(dpy, DefaultScreen(dpy));
	fg = WhitePixel(dpy, DefaultScreen(dpy));

	/* thread_setconcurrency(3); */
	line_t=thread_create(NULL, 0, line_thread, (void *)1, 0);
	rectangle_t=thread_create(NULL, 0, rectangle_thread, (void *)2, 0);
	circle_t=thread_create(NULL, 0, circle_thread, (void *)3, 0);

	XtMainLoop();

}


Boolean
use_color()
{
	static Boolean set;
	static Arg arg[] = {
		{ XtNset, (XtArgVal) &set },
	};
	XtGetValues(color_item, arg, XtNumber(arg));
	return(set);
}


Boolean
use_fill()
{
	static Boolean set;
	static Arg arg[] = {
		{ XtNset, (XtArgVal) &set },
	};
	XtGetValues(fill_item, arg, XtNumber(arg));
	return(set);
}


void
set_string(widget, str)
	Widget widget;
	char *str;
{
	Arg arg;

	XtSetArg(arg, XtNstring, str);
	XtSetValues(widget, &arg, 1);
}


void
line_thread(which)
	int             which;
{
	GC              thread_gc;
	int             x0, y0, x1, y1;
	int             lines = 0;
	char            linebuff[20];

	thread_gc = XCreateGC(dpy, win, (unsigned long) 0, (XGCValues *) 0);

	while (1) {
		random_coord(&x0, &y0);
		random_coord(&x1, &y1);

		if (use_color())
			XSetForeground(dpy, thread_gc, color());
		else
			XSetForeground(dpy, thread_gc, bg);

		XDrawLine(dpy, win, thread_gc, x0, y0, x1, y1);

		sprintf(linebuff, "Lines: %d", ++lines);
		set_string(linemessage, linebuff);

		XFlush(dpy);

		/* sleep(sleep_time); */
		poll(NULL, 0, sleep_time * 100);
	}
}


#ifndef MIN
#define MIN(a,b) (((a) < (b)) ? (a) : (b))
#define MAX(a,b) (((a) > (b)) ? (a) : (b))
#endif


void
rectangle_thread(which)
	int             which;
{
	GC              thread_gc;
	int             x0, y0, x1, y1, w, h;
	int             boxes = 0;
	char            boxbuff[20];

	thread_gc = XCreateGC(dpy, win, (unsigned long) 0, (XGCValues *) 0);

	while (1) {
		random_coord(&x0, &y0);
		random_coord(&x1, &y1);

		w = MAX(x0, x1) - MIN(x0, x1);
		h = MAX(y0, y1) - MIN(y0, y1);


		if (use_color())
			XSetForeground(dpy, thread_gc, color());
		else
			XSetForeground(dpy, thread_gc, bg);


		if (use_fill())
			XFillRectangle(dpy, win, thread_gc, 
				MIN(x0, x1), MIN(y0, y1), 
				w, h);
		else
			XDrawRectangle(dpy, win, thread_gc, 
				MIN(x0, x1), MIN(y0, y1), 
				w, h);

		sprintf(boxbuff, "Boxes: %d", ++boxes);
		set_string(rectanglemessage, boxbuff);

		XFlush(dpy);

		/* sleep(sleep_time); */
		poll(NULL, 0, sleep_time * 100);
	}

}


void
circle_thread(which)
	int             which;
{
	GC              thread_gc;
	int             x0, y0, x1, y1, w, h;
	int             circles = 0;
	char            circlebuff[20];

	thread_gc = XCreateGC(dpy, win, (unsigned long) 0, (XGCValues *) 0);

	while (1) {
		random_coord(&x0, &y0);
		random_coord(&x1, &y1);

		w = MAX(x0, x1) - MIN(x0, x1);
		h = MAX(y0, y1) - MIN(y0, y1);


		if (use_color())
			XSetForeground(dpy, thread_gc, color());
		else
			XSetForeground(dpy, thread_gc, bg);

		if (use_fill())
			XFillArc(dpy, win, thread_gc, 
				MIN(x0, x1), MIN(y0, y1),
				w, h, 0, 360 * 64);
		else
			XDrawArc(dpy, win, thread_gc, 
				MIN(x0, x1), MIN(y0, y1),
				w, h, 0, 360 * 64);

		sprintf(circlebuff, "Circles: %d", ++circles);
		set_string(circlemessage, circlebuff);

		XFlush(dpy);
		poll(NULL, 0, sleep_time * 100);
	}

}


void
random_coord(x, y)
	int *x;
	int *y;
{
	static Dimension width, height;
	static Arg arg[] = {
		{ XtNwidth, (XtArgVal) &width},
		{ XtNheight, (XtArgVal) &height}
	};

	XtGetValues(canvas, arg, XtNumber(arg));
	*x = rand() % (int) width;
	*y = rand() % (int) height;
}


int color()
{
	return(rand() % 255);
}


void
quit_callback(w, client_data, call_data)
  Widget     w;
  XtPointer  client_data, call_data;
{
	XtDestroyWidget(toplevel);
	exit(0);
}


void
change_sleep_time(w, client_data, call_data)
	Widget w;
	XtPointer client_data;
	XtPointer call_data;
{
	sleep_time = *(int*) call_data;
}

void
slider_moved(w, client_data, call_data)
	Widget w;
	XtPointer client_data;
	XtPointer call_data;
{
	Widget	label_widget = (Widget) client_data;
	Arg  	arg;
	char	str[10];

	sprintf(str, "%3d", *(int*)call_data);
	XtSetArg(arg, XtNstring, (XtArgVal) str);
	XtSetValues(label_widget, &arg, 1);
}


Widget
create_slider(name, parent, args, nargs)
	char *name;
	Widget parent;
	Arg *args;
	int nargs;
{
	Widget	tmp;
	Widget	ca;
	Widget	valuew;

	ca = XtCreateManagedWidget(name, controlAreaWidgetClass,
				parent, NULL, 0);

	(void) XtCreateManagedWidget("sliderlabel", staticTextWidgetClass,
				ca, NULL, 0);

	valuew = XtCreateManagedWidget("slidervalue", staticTextWidgetClass,
				ca, NULL, 0);

	tmp = XtCreateManagedWidget(name, sliderWidgetClass,
				ca, NULL, 0);

	XtAddCallback(tmp, XtNsliderMoved, slider_moved, (XtPointer) valuew);

	return ca;
}

