 
/* 
 *      (c) Copyright 1991 Sun Microsystems, Inc. Sun design patents 
 *      pending in the U.S. and foreign countries. See LEGAL NOTICE 
 *      file for terms of the license.
 */

#include <sys/types.h>
#include <thread.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include "fifo.h"
#include "XtToken.h"
#include <X11/Intrinsic.h>


typedef struct token {
	fifo_t		fifo;
	int		pipes[2];
	XtInputId	input_id;
} token_t;


static token_t _XtToken;
static int ok_to_lock = 0;


void 
_XtTokenDispatch(client_data, source, id)
	XtPointer client_data;
	int *source;
	XtInputId *id;
{
	fifo_t * fifo = & ((token_t *)client_data)->fifo;

	/* schedule to continue after 5 (max) threads have been given token */

	fifo_yield(fifo, 5);
}


static void
_XtBeforeBlocking(token)
	token_t *token;
{
	write(token->pipes[1], " ", 1);
/*
	      &token, sizeof(token_t *));
*/
}


void
XtInitializeToken()
{
	if (!ok_to_lock) {
		/*
		 * Race to this point not considered a problem, since the
		 * semantics of calling toolkit initialization routines
		 * more than once are undefined.
		 */
		ok_to_lock = 1; 

		if (pipe(_XtToken.pipes) != 0)
			perror("XtInitializeToken:pipe:");

		fifo_init(&_XtToken.fifo, 0, 0);

		/*
		 * The fifo "token" is held on return.  This is probably not
		 * necessary, but it corresponds to what the documentation
		 * says.  Don't involve the dispatcher because it is not
		 * set up yet.
		 */
		fifo_enter(&_XtToken.fifo, NULL, NULL);
	}
}


void
XtInitializeTokenDispatch()
{
	/*
	 * Must be called after XtInitializeToken() is called.
	 */
	if (ok_to_lock) {
		/*
		 * Race to this point not considered a problem, since the
		 * semantics of calling toolkit initialization routines
		 * more than once are undefined.
		 */
		_XtToken.input_id = XtAppAddInput(
						_XtDefaultAppContext(),
						_XtToken.pipes[0],
						XtInputReadMask,
						_XtTokenDispatch,
						&_XtToken);
	}
}


void
XtGetToken()
{
	if (ok_to_lock)	{
		/*
		fifo_t         *buff;
		*/
		char		buff;

		if (fifo_tryenter(&_XtToken.fifo))
			return;

		fifo_enter(&_XtToken.fifo, _XtBeforeBlocking, &_XtToken);
	
		read(_XtToken.pipes[0], &buff, 1);
			/*sizeof(token_t *));*/
	}
}


void
XtReleaseToken()
{
	if (ok_to_lock)
		fifo_exit(&_XtToken.fifo);
}


LockDisplay()
{
	if (ok_to_lock)
		XtGetToken();
}


UnlockDisplay()
{
	if (ok_to_lock)
		XtReleaseToken();
}


LockMutex(mutex)
	void **mutex;
{
	static mutex_t init_lock;

	if (!*mutex) {
		mutex_enter(&init_lock);
		if (!*mutex) {	/* retest because of race */
			mutex_t        *tmp;
			tmp = (mutex_t *) malloc(sizeof(mutex_t));
			mutex_init(tmp, 0, 0);
			*mutex = (void *) tmp;
		}
		mutex_exit(&init_lock);
	}
	mutex_enter(*((mutex_t **) mutex));
}


UnlockMutex(mutex)
	void **mutex;
{
	assert(mutex);
	mutex_exit(*((mutex_t **) mutex));
}

