/*
	This program is a simple hello world demo to
	demonstrate using threads in a xview application.

	3 threads are created to render onto a canvas. Each thread
	is repsonsible for drawing it's own shape (circle, line, rectangle)

	This program runs with a sample mt safe xview library and a sample
	thread library under alpha Solaris 2.0
*/
#include <stdio.h>
#include <errno.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <xview/xview.h>
#include <xview/frame.h>
#include <xview/panel.h>
#include <thread.h>

#ifndef MIN
#define MIN(a,b) (((a) < (b)) ? (a) : (b))
#define MAX(a,b) (((a) > (b)) ? (a) : (b))
#endif

Frame frame;
Panel_item sleep_item;
Panel_item fill_item;
Panel_item color_item;
Panel_item linemessage, rectanglemessage, circlemessage;
Canvas canvas;
Window win;
Display * dpy;
unsigned long	fg, bg, bd;

void quit();
void rectangle_thread();
void circle_thread();
void line_thread();
void random_coord();

thread_id_t line_t, rectangle_t, circle_t;

main(argc, argv)
int argc;
char *argv[];
{
  Panel panel;

  xv_init (XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

  frame=xv_create(NULL, FRAME, 
		  FRAME_LABEL, "SvR4 MT Xview",
		  XV_WIDTH, 650,
		  XV_HEIGHT, 200,
		  NULL);


  panel = (Panel)xv_create(frame, PANEL, 
			   XV_HEIGHT, 70,
			   NULL);
 
  xv_create(panel, PANEL_BUTTON, 
		   PANEL_LABEL_STRING, "Quit",
		   PANEL_NOTIFY_PROC, quit,
		   NULL);

  sleep_item = (Panel_item)xv_create(panel, PANEL_SLIDER,
	           PANEL_VALUE, 1,
	           PANEL_MIN_VALUE, 1,
	           PANEL_MAX_VALUE, 10,
	           PANEL_LABEL_STRING, "Sleep Value: ",
	           NULL);

  fill_item = (Panel_item)xv_create(panel, PANEL_CHOICE,
	           PANEL_VALUE, 0,
		   PANEL_CHOICE_STRINGS, "Yes", "No", NULL,
	           PANEL_LABEL_STRING, "Fill?: ");

  color_item = (Panel_item)xv_create(panel, PANEL_CHOICE,	
	           PANEL_VALUE, 0,
		   PANEL_CHOICE_STRINGS, "Yes", "No", NULL,
	           PANEL_LABEL_STRING, "Color?: ",
	           NULL);

  linemessage = (Panel_item)xv_create(panel, PANEL_MESSAGE,
		   XV_X, 5,
		   XV_Y, 55,
		   PANEL_LABEL_STRING, "Lines: 0");

  rectanglemessage = (Panel_item)xv_create(panel, PANEL_MESSAGE,
		   XV_X, 315,
		   PANEL_LABEL_STRING, "Boxes: 0");

  circlemessage = (Panel_item)xv_create(panel, PANEL_MESSAGE,
		   XV_X, 525,
		   PANEL_LABEL_STRING, "Circles: 0");

  canvas = (Canvas)xv_create(frame, CANVAS, 
			    WIN_BELOW, panel, 
			    WIN_SHOW, TRUE, 
			    NULL);

  dpy = (Display *)xv_get(frame, XV_DISPLAY);
  win = (Window)   xv_get(canvas_paint_window(canvas), XV_XID);
  
  bg = BlackPixel(dpy, DefaultScreen(dpy));
  fg = WhitePixel(dpy, DefaultScreen(dpy));

  window_fit (frame);

  thread_setconcurrency(3);
  line_t=thread_create(NULL, 0, line_thread, 1, 0);

  rectangle_t=thread_create(NULL, 0, rectangle_thread, 2, 0);

  circle_t=thread_create(NULL, 0, circle_thread, 3, 0);

  xv_main_loop(frame);

  exit(0);
}

void
  line_thread(which)
int which;
{
  GC thread_gc;
  int x0, y0, x1, y1;
  int lines=0;
  char linebuff[20];
  
  thread_gc = XCreateGC(dpy, win, (unsigned long) 0, (XGCValues*) 0);
  
  while(1) {
    random_coord(&x0, &y0);
    random_coord(&x1, &y1);
    
    if ((int)xv_get(color_item, PANEL_VALUE))
      XSetForeground(dpy, thread_gc, bg);
    else
      XSetForeground(dpy, thread_gc, color());
    
    XDrawLine(dpy, win, thread_gc, x0, y0, x1, y1);

    sprintf(linebuff, "Lines: %d", ++lines);
    xv_set(linemessage, PANEL_LABEL_STRING, linebuff, 0);
    
    XFlush(dpy);

    poll (NULL, 0, (int)xv_get(sleep_item, PANEL_VALUE)*100);

    XSetForeground(dpy, thread_gc, fg);
    
    XDrawLine(dpy, win, thread_gc, x0, y0, x1, y1);
  }
}


void
  rectangle_thread(which)
int which;
{
  GC thread_gc;
  int x0, y0, x1, y1, w, h, reset;
  int boxes=0;
  char boxbuff[20];
  
  thread_gc = XCreateGC(dpy, win, (unsigned long) 0, (XGCValues*) 0);
  
  while(1) {
    random_coord(&x0, &y0);
    random_coord(&x1, &y1);
    
    w = MAX(x0, x1) - MIN(x0, x1);
    h = MAX(y0, y1) - MIN(y0, y1);
    
    
    if ((int)xv_get(color_item, PANEL_VALUE))
      XSetForeground(dpy, thread_gc, bg);
    else
      XSetForeground(dpy, thread_gc, color());
    
    if ((reset=(int)xv_get(fill_item, PANEL_VALUE)))
      XDrawRectangle(dpy, win, thread_gc, MIN(x0, x1), MIN(y0, y1), w, h);
    else
      XFillRectangle(dpy, win, thread_gc, MIN(x0, x1), MIN(y0, y1), w, h);

    sprintf(boxbuff, "Boxes: %d", ++boxes);
    xv_set(rectanglemessage, PANEL_LABEL_STRING, boxbuff, 0);

    XFlush(dpy);

    poll (NULL, 0, (int)xv_get(sleep_item, PANEL_VALUE)*100);
    
    XSetForeground(dpy, thread_gc, fg);
    
    if (reset)
      XDrawRectangle(dpy, win, thread_gc, MIN(x0, x1), MIN(y0, y1), w, h);
    else
      XFillRectangle(dpy, win, thread_gc, MIN(x0, x1), MIN(y0,y1), w, h);
  }
  
}

void
  circle_thread(which)
int which;
{
  GC thread_gc;
  int x0, y0, x1, y1, w, h, reset;
  int circles=0;
  char circlebuff[20];
  
  thread_gc = XCreateGC(dpy, win, (unsigned long) 0, (XGCValues*) 0);
  
  while(1) {
    random_coord(&x0, &y0);
    random_coord(&x1, &y1);
    
    w = MAX(x0, x1) - MIN(x0, x1);
    h = MAX(y0, y1) - MIN(y0, y1);
    
    
    if ((int)xv_get(color_item, PANEL_VALUE))
      XSetForeground(dpy, thread_gc, bg);
    else
      XSetForeground(dpy, thread_gc, color());

    if ((reset=(int)xv_get(fill_item, PANEL_VALUE)))
      XDrawArc(dpy, win, thread_gc, MIN(x0, x1), MIN(y0, y1), 
	     w,h,0, 360*64);
    else
      XFillArc(dpy, win, thread_gc, MIN(x0, x1), MIN(y0, y1), 
	     w,h,0, 360*64);

    sprintf(circlebuff, "Circles: %d", ++circles);
    xv_set(circlemessage, PANEL_LABEL_STRING, circlebuff, 0);
    
    XFlush(dpy);
    poll (NULL, 0, (int)xv_get(sleep_item, PANEL_VALUE)*100);
        
    XSetForeground(dpy, thread_gc, fg);

    if (reset)
      XDrawArc(dpy, win, thread_gc, MIN(x0, x1), MIN(y0, y1), 
	     w,h,0, 360*64);
    else
      XFillArc(dpy, win, thread_gc, MIN(x0, x1), MIN(y0, y1), 
	     w,h,0, 360*64);
  }
  
}


void
  random_coord(x, y)
int *x;
int *y;
{
  *x = rand() % (int)xv_get(canvas, XV_WIDTH);
  *y = rand() % (int)xv_get(canvas, XV_HEIGHT);
}

int color()
{
  return(rand() % 255);
}

void
quit()
{
  notify_stop();
}

/*
	These symbols are here to get around some problems with the
	library.
*/
fd_set svc_fdset;

int
svc_getreqset()
{
}
