/*
  a set of routines to implement a single token system to protect the
  notifier and xview routines from re-entrant calls.

  barts 7/19/91

  */
#include <stdio.h>
#include <sys/time.h>
#include <thread.h>
#include <unistd.h>
#include <errno.h>
#include "fifo.h"
#include "token.h"

/*
  token_init initializes the token for use with the XView notifier
  */

void token_init(token_t * token)
{
  static void notifier_token_dispatch();

  if(pipe(token->notifier_pipes) != 0)		/* build a pipe for letting
						   the notifier know we want the
						   token */
						   
    perror("token_init:pipe:");

  fifo_init(& token->fifo, 0, 0);
  fifo_enter(&token->fifo);

  notify_set_input_func(token, 
			notifier_token_dispatch,
			token->notifier_pipes[0]);
}

/*
  this routine acquires the token... used only by client threads
  */

void token_enter(token_t * token)
{
  fifo_t * fifo = & (token->fifo);
  fifo_t * buff;

  thread_id_t xv_get_notifier_thread();
  
  void signal_notifier();

  if(fifo_signal_enter(fifo, signal_notifier, token))	/* enter the fifo....*/
    if(read(token->notifier_pipes[0],			/* remove bytes if we wrote them */
	    & buff, 
	    sizeof(token_t * )) != 4)
      fprintf(stderr, "token_enter: couldn't read 4 bytes\n");
}

static void signal_notifier(token_t * token)
{
  if(write(token->notifier_pipes[1],			/* let the notifier know */
	& token, 
	sizeof(token_t * )) != 4)
    fprintf(stderr,"signal_notifier:Couldn't write 4 bytes\n");
}
  
void token_exit(token_t * token)
{
  fifo_exit(&token->fifo);
}

/*
  this routine releases the token to any pending threads and then attempts
  to reacquire it.  Used only by the notifier.  The scheduling algorithm is
  subject to change 
  */

static void notifier_token_dispatch(void * client, int desc)
{
  fifo_t * fifo = & ((token_t *)client)->fifo; 

  /* max others we let in
     before letting the entire notifier loop run
     */

  int i = 5;
  
  while(i-- && fifo_num_waiters(fifo)) {
    fifo_exit(fifo);
    fifo_grab(fifo);
  }
}
