/*
  a set of routines to implement a single token system to protect the
  notifier and xview routines from re-entrant calls.

  barts 7/19/91

  */

#include <sys/types.h>
#include <thread.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include "fifo.h"
#include "token.h"

static token_t xv_token;

static mutex_t init_lock;

static int ok_to_lock;

static thread_id_t notifier_thread;


xv_init_token()	/* called by notifier thread early on */
{
  if(!ok_to_lock)
    {
      token_init(& xv_token);
      ok_to_lock = 1;
      notifier_thread = thread_get_id();
    }
}

thread_id_t xv_get_notifier_thread()
{
  return(notifier_thread);
}

xv_get_token()
{
  token_enter(& xv_token);
}

xv_release_token()
{
  token_exit(& xv_token);
}

LockMutex(void ** mutex)
{
  if(! * mutex)
    {
      mutex_enter(&init_lock);
      if(! *mutex) /* retest because of race */
	{
	  mutex_t * tmp;
	  tmp = (mutex_t *)  malloc(sizeof(mutex_t));
	  mutex_init(tmp, 0, 0);
	  *mutex = (void *) tmp;
	}
      mutex_exit(&init_lock);
    }

  mutex_enter(*((mutex_t **) mutex));
}	

UnlockMutex(void ** mutex)
{
  assert(mutex);
  mutex_exit(* ((mutex_t **) mutex));
}

LockDisplay()
{
  if(ok_to_lock)
    xv_get_token();
}

UnlockDisplay()
{
  if(ok_to_lock)
    xv_release_token();
}


